//**************************************************************************************************
// Filename							MT_WH1602.c
// Platform     				STM32F0x1/STM32F10x/STM32F4xx
// Author								mail_robot
// Current version			2.1.0
// Date         				11.01.2017
//**************************************************************************************************

#include "MT_WH1602.h"

//**************************************************************************************************
// Function      MT_WH1602_Init()
// Description   Initalization WH1602 module
// Parameters    None
// RetVal	     None
//**************************************************************************************************

void MT_WH1602_Init()
{
	HAL_Delay(50);
	HAL_GPIO_WritePin(MT_WH1602_RS_PORT, MT_WH1602_RS_PIN, GPIO_PIN_RESET);
#if RW_PIN_USE
	HAL_GPIO_WritePin(MT_WH1602_RW_PORT, MT_WH1602_RW_PIN, GPIO_PIN_RESET);
#endif
	
	HAL_GPIO_WritePin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB6_PORT, MT_WH1602_DB6_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB5_PORT, MT_WH1602_DB5_PIN, GPIO_PIN_SET);
	HAL_GPIO_WritePin(MT_WH1602_DB4_PORT, MT_WH1602_DB4_PIN, GPIO_PIN_SET);
	MT_WH1602_DataReadWrite();
	HAL_Delay(5);
	HAL_GPIO_WritePin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB6_PORT, MT_WH1602_DB6_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB5_PORT, MT_WH1602_DB5_PIN, GPIO_PIN_SET);
	HAL_GPIO_WritePin(MT_WH1602_DB4_PORT, MT_WH1602_DB4_PIN, GPIO_PIN_SET);
	MT_WH1602_DataReadWrite();
	HAL_Delay(1);
	HAL_GPIO_WritePin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB6_PORT, MT_WH1602_DB6_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB5_PORT, MT_WH1602_DB5_PIN, GPIO_PIN_SET);
	HAL_GPIO_WritePin(MT_WH1602_DB4_PORT, MT_WH1602_DB4_PIN, GPIO_PIN_SET);
	MT_WH1602_DataReadWrite();
	HAL_Delay(1);
	
	// set interface length
#if _4_BIT_MODE
	#warning ("Warning: data bus length is setting to 4 bits mode!")
	HAL_GPIO_WritePin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB6_PORT, MT_WH1602_DB6_PIN, GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB5_PORT, MT_WH1602_DB5_PIN, GPIO_PIN_SET);
	HAL_GPIO_WritePin(MT_WH1602_DB4_PORT, MT_WH1602_DB4_PIN, GPIO_PIN_RESET);
	MT_WH1602_DataReadWrite();
	HAL_Delay(1);
	MT_WH1602_WriteData(FUNCTION_SET | MT_N, MT_WH1602_COMMAND); 	//0b00101000 = 0x28
	HAL_Delay(1);
#else
	#warning ("Warning: data bus length is setting to 8 bits mode!")
	MT_WH1602_WriteData(FUNCTION_SET | MT_DL | MT_N, MT_WH1602_COMMAND); 	// 0b00111000 = 0x38
	HAL_Delay(1);
#endif
	
	MT_WH1602_WriteData(DISPLAY_ONOFF | MT_D, MT_WH1602_COMMAND);	HAL_Delay(1); // 0b00001100 = 0x0C
	MT_WH1602_WriteData(CLEAR_DISPLAY, MT_WH1602_COMMAND);	HAL_Delay(1);
	MT_WH1602_WriteData(ENTRY_MODE_SET | MT_ID, MT_WH1602_COMMAND); HAL_Delay(1);// 0b00000111 = 0x07
}


void lcd_write_str(char *str) {
    do {
    	MT_WH1602_WriteData(*str, MT_WH1602_DATA);
		#if RW_PIN_USE
    	while (MT_WH1602_ReadBusy()!=FALSE);
		#else
			MT_WH1602_Delay(MT_WH1602_DATA_RW_DELAY);
		#endif
    }while(*++str);
}

//**************************************************************************************************
// Function      MT_WH1602_Delay()
// Description   Delay function
// Parameters    time - number of microseconds to wait
// RetVal	     	 None
//**************************************************************************************************
void MT_WH1602_Delay(uint32_t us) 
{
  volatile uint32_t i = (HAL_RCC_GetSysClockFreq()/10000000)*us;	
  for (; i != 0; i--);
}


//**************************************************************************************************
// Function      MT_WH1602_WriteData()
// Description   Write data to CGRAM or DDRAM
// Parameters    data - 8-bit data byte
// RetVal	       None
//**************************************************************************************************
void MT_WH1602_WriteData(uint8_t data, uint8_t type)
{	
	HAL_GPIO_WritePin(MT_WH1602_RS_PORT, MT_WH1602_RS_PIN, type ? GPIO_PIN_SET : GPIO_PIN_RESET);
#if RW_PIN_USE
	HAL_GPIO_WritePin(MT_WH1602_RW_PORT, MT_WH1602_RW_PIN, GPIO_PIN_RESET);
#endif
	
	HAL_GPIO_WritePin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN, data & BIT_7_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
  HAL_GPIO_WritePin(MT_WH1602_DB6_PORT, MT_WH1602_DB6_PIN, data & BIT_6_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB5_PORT, MT_WH1602_DB5_PIN, data & BIT_5_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB4_PORT, MT_WH1602_DB4_PIN, data & BIT_4_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
#if _4_BIT_MODE
	MT_WH1602_DataReadWrite();
	HAL_GPIO_WritePin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN, data & BIT_3_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
  HAL_GPIO_WritePin(MT_WH1602_DB6_PORT, MT_WH1602_DB6_PIN, data & BIT_2_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB5_PORT, MT_WH1602_DB5_PIN, data & BIT_1_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB4_PORT, MT_WH1602_DB4_PIN, data & BIT_0_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
#else	
	HAL_GPIO_WritePin(MT_WH1602_DB3_PORT, MT_WH1602_DB3_PIN, data & BIT_3_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);	
	HAL_GPIO_WritePin(MT_WH1602_DB2_PORT, MT_WH1602_DB2_PIN, data & BIT_2_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB1_PORT, MT_WH1602_DB1_PIN, data & BIT_1_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_DB0_PORT, MT_WH1602_DB0_PIN, data & BIT_0_MASK ? GPIO_PIN_SET : GPIO_PIN_RESET);
#endif
	MT_WH1602_DataReadWrite();
} // End of function MT_WH1602_WriteData()

//**************************************************************************************************
// Function      MT_WH1602_DataReadWrite()
// Description   Strob impulse for data read/write
// Parameters    None
// RetVal	     	 None
//**************************************************************************************************
void MT_WH1602_DataReadWrite()
{
	HAL_GPIO_WritePin(MT_WH1602_E_PORT, MT_WH1602_E_PIN, GPIO_PIN_SET);
	//HAL_Delay(1);
	MT_WH1602_Delay(MT_WH1602_DATA_RW_DELAY);
	HAL_GPIO_WritePin(MT_WH1602_E_PORT, MT_WH1602_E_PIN, GPIO_PIN_RESET);
} // End of function MT_WH1602_DataReadWrite()

//**************************************************************************************************
// Function      MT_WH1602_ReadBusy()
// Description   Read busy flag and address
// Parameters    None
// RetVal	       busyFlag - busy flag current value
//**************************************************************************************************
bool MT_WH1602_ReadBusy()
{
#if RW_PIN_USE
	bool busyFlag;
	HAL_GPIO_WritePin(MT_WH1602_RS_PORT,  MT_WH1602_RS_PIN,  GPIO_PIN_RESET);
	HAL_GPIO_WritePin(MT_WH1602_RW_PORT, MT_WH1602_RW_PIN, GPIO_PIN_SET);
  
	// DB7 pin is input	
	GPIO_InitStruct.Mode  = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pin   = MT_WH1602_DB7_PIN;
	HAL_GPIO_Init(MT_WH1602_DB7_PORT, &GPIO_InitStruct);
	
	HAL_GPIO_WritePin(MT_WH1602_E_PORT, MT_WH1602_E_PIN, GPIO_PIN_SET);
	MT_WH1602_Delay(MT_WH1602_DATA_RW_DELAY);
	busyFlag = (bool)HAL_GPIO_ReadPin(MT_WH1602_DB7_PORT, MT_WH1602_DB7_PIN);
	HAL_GPIO_WritePin(MT_WH1602_E_PORT, MT_WH1602_E_PIN, GPIO_PIN_RESET);
	
	// DB7 pin is output
	GPIO_InitStruct.Mode  = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pin   = MT_WH1602_DB7_PIN;
	HAL_GPIO_Init(MT_WH1602_DB7_PORT, &GPIO_InitStruct);
	
	return busyFlag;
#else
	return 1;
#endif
} // End of function MT_WH1602_ReadBusy()



//****************************************** end of file *******************************************
