//**************************************************************************************************
// Filename					MT_WH1602.h
// Platform     			STM32F10x/STM32F4xx
// Author					mail_robot
// Current version  		2.1.0
// Date         			11.01.2017
//**************************************************************************************************

//    
#ifndef MT_WH1602_H
#define MT_WH1602_H

//**************************************************************************************************
// Choose your platform
//**************************************************************************************************

#define STM32F10x											(0)
#define STM32F4xx											(1)
#define STM32F0xx											(2)

#define PLATFORM											(STM32F0xx)

//**************************************************************************************************
// Includes
//**************************************************************************************************

#if (PLATFORM == STM32F10x)
// Get RCC functions from standard peripheral library
#include "stm32f1xx_hal.h"
#endif

#if (PLATFORM == STM32F4xx)
// Get RCC functions from standard peripheral library
#include "stm32f4xx_hal.h"
#endif

#if (PLATFORM == STM32F0xx)
// Get RCC functions from standard peripheral library
#include "stm32f0xx_hal.h"
#endif

//**************************************************************************************************
// Declarations and definitions
//**************************************************************************************************

#include <stdbool.h> 			//   bool

#define FALSE (false)
#define TRUE (!FALSE)

#if (PLATFORM == STM32F4xx)
// Define TRUE, FALSE and bool if not defined
#if (BOOL_IS_DEFINED == 0)
//typedef enum {FALSE = 0, TRUE = !FALSE} bool;
#define BOOL_IS_DEFINED								(1)
#endif // #if (BOOL_IS_DEFINED == 0)
#endif // #if (PLATFORM == STM32F4xx)

#if (PLATFORM == STM32F0xx)
// Define TRUE, FALSE and bool if not defined
#ifndef BOOL_IS_DEFINED
//typedef enum {FALSE, TRUE} bool;
#define BOOL_IS_DEFINED								(1)
#endif // #if (BOOL_IS_DEFINED == 0)
#endif // #if (PLATFORM == STM32F051R8)

#define _4_BIT_MODE									(TRUE)
#define RW_PIN_USE									(FALSE)

// Select which pins are connected to the WH1602 module

// Set RS port
#define MT_WH1602_RS_PORT							(GPIOA)
#define MT_WH1602_RS_PIN							(GPIO_PIN_3)

#if RW_PIN_USE
// Set RW port
#define MT_WH1602_RW_PORT							(GPIOA)	
#define MT_WH1602_RW_PIN							(GPIO_PIN_6)
#endif

// Set E port
#define MT_WH1602_E_PORT							(GPIOA)	
#define MT_WH1602_E_PIN								(GPIO_PIN_4)
// Set DB7 port
#define MT_WH1602_DB7_PORT						(GPIOB)
#define MT_WH1602_DB7_PIN	 						(GPIO_PIN_1)
// Set DB6 port
#define MT_WH1602_DB6_PORT						(GPIOA)
#define MT_WH1602_DB6_PIN							(GPIO_PIN_7)
// Set DB5 port
#define MT_WH1602_DB5_PORT						(GPIOA)
#define MT_WH1602_DB5_PIN							(GPIO_PIN_6)
// Set DB4 port
#define MT_WH1602_DB4_PORT						(GPIOA)
#define MT_WH1602_DB4_PIN							(GPIO_PIN_5)

#if _4_BIT_MODE
#else
// Set DB3 port
#define MT_WH1602_DB3_PORT							(GPIOC)
#define MT_WH1602_DB3_PIN							(GPIO_PIN_12)
// Set DB2 port
#define MT_WH1602_DB2_PORT							(GPIOD)
#define MT_WH1602_DB2_PIN							(GPIO_PIN_2)
// Set DB1 port
#define MT_WH1602_DB1_PORT							(GPIOB)
#define MT_WH1602_DB1_PIN							(GPIO_PIN_3)
// Set DB0 port
#define MT_WH1602_DB0_PORT							(GPIOB)
#define MT_WH1602_DB0_PIN							(GPIO_PIN_4)
#endif

// Delay us function constants
#define MT_WH1602_DELAY_uS							(24000)
// Delay value for strob impulse
#define MT_WH1602_DATA_RW_DELAY						(1)

// Bit masks for different bits in byte
#define BIT_7_MASK										(0x80)
#define BIT_6_MASK										(0x40)
#define BIT_5_MASK										(0x20)
#define BIT_4_MASK										(0x10)
#define BIT_3_MASK										(0x08)
#define BIT_2_MASK										(0x04)
#define BIT_1_MASK										(0x02)
#define BIT_0_MASK										(0x01)

#define CLEAR_DISPLAY									(0x01)
#define RETURN_HOME										(0x02)
#define	ENTRY_MODE_SET									(0x04)
#define DISPLAY_ONOFF									(0x08)
#define CURSOR_SHIFT									(0x10)
#define FUNCTION_SET									(0x20)
#define SET_SGRAM_ADDR									(0x40)
#define SET_DDRAM_ADDR									(0x80)
#define MT_ID											(0x02)
#define MT_S											(0x01)
#define MT_D											(0x04)
#define MT_C											(0x02)
#define MT_B											(0x01)
#define MT_SC											(0x08)
#define MT_RL											(0x04)
#define MT_N											(0x08)
#define MT_F											(0x04)
#define MT_DL											(0x10)



#define MT_WH1602_COMMAND								(0x00)
#define MT_WH1602_DATA									(0x01)

// MT_WH1602_Library function prototypes
extern void lcd_write_str(char*str);
extern void MT_WH1602_Init(void);
extern void MT_WH1602_Delay(uint32_t time);
extern void MT_WH1602_DataReadWrite(void);
extern bool MT_WH1602_ReadBusy(void);
extern void MT_WH1602_SetAddr(uint8_t addr);
extern void MT_WH1602_WriteData(uint8_t data, uint8_t type);

#endif

//****************************************** end of file *******************************************
