
#include "main.h"
#include "stm32f0xx_hal.h"
#include "MT_WH1602.h"

I2C_HandleTypeDef hi2c1;

void SystemClock_Config(void);
void Error_Handler(void);
static void MX_GPIO_Init(void);
static void MX_I2C1_Init(void);

void Timer_read(void);
void TimerWrite(uint8_t, uint8_t);

void DisplayTime(void);
void DisplayTimeset(void);
void KeyboardScan(void);
void MenuExec(void);
void EventExec(void);
void EditTime (void);
void EditTimeset(uint8_t *Timeset);
void SetValue(uint8_t* val, uint8_t lower, uint8_t upper);
void TimeBlink(void);
void RestoreTimeSet(void);
void AlarmCheck(void);
void RelayOnOff(uint16_t, uint16_t, uint16_t, uint8_t);

uint8_t TimerData[8] = {0,0,0,0,0,0,0,0};
uint8_t DisplayTimeData[7];
uint8_t Timeset1[5];
uint8_t Timeset2[5];
uint8_t KeyUpEvent, KeyDownEvent, KeySelectEvent, LongKeySelectEvent;
uint8_t CursorPosition, OldCursorPosition;
uint8_t EditModeItem;
uint8_t EditModeCursor;
uint8_t ForceRedraw;
uint8_t Blink;

#define BUTTON_TIMEOUT 20;
#define LONG_PRESS_TIMEOUT 3000;
#define BLINK_TIMEOUT 250;

int main(void)
{
  HAL_Init();
  SystemClock_Config();
  MX_GPIO_Init();
  MX_I2C1_Init();
	MT_WH1602_Init();

	Timer_read();
	if (TimerData[0] & 0x80) TimerWrite(0x00, TimerData[0]&0x7F); // reset start bit if not reset
	if (TimerData[2] & 0x40) TimerWrite(0x02, TimerData[2]&(0x3F|TimerData[2])); // reset 12/24 bit if not reset
	
	RestoreTimeSet();
	
  while (1)
  {
		KeyboardScan();
		EventExec();
		if (CursorPosition != OldCursorPosition) MenuExec();
		if (EditModeItem > 1) DisplayTimeset();
		if (EditModeItem != 1) Timer_read();
		if (!CursorPosition) DisplayTime();
		if (!EditModeItem) AlarmCheck();	
  }
}

void RelayOnOff(uint16_t c_point, uint16_t p_ON, uint16_t p_OFF, uint8_t Channel)
{
	if (p_ON > p_OFF)
	{
		if ((c_point >= p_OFF) && (c_point < p_ON)) HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_RESET);
		else HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_SET);
	}
	else HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_RESET);
	if (p_ON < p_OFF)
	{
		if ((c_point >= p_ON) && (c_point < p_OFF)) HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_SET);
		else HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_RESET);
	}
	else HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_RESET);
	if (p_ON == p_OFF)
	{
		HAL_GPIO_WritePin(GPIOF, Channel, GPIO_PIN_RESET);
	}
}

void AlarmCheck(void)
{
	static uint16_t AlarmPoint1_ON, AlarmPoint1_OFF, AlarmPoint2_ON, AlarmPoint2_OFF, CurrentTimePoint;

	CurrentTimePoint = DisplayTimeData[0]*60+ DisplayTimeData[1];
	
	if (Timeset1[4])
	{
		AlarmPoint1_ON = Timeset1[0]*60+Timeset1[1];
		AlarmPoint1_OFF = Timeset1[2]*60+Timeset1[3];
		RelayOnOff(CurrentTimePoint, AlarmPoint1_ON, AlarmPoint1_OFF, 2);
	}
	
	if (Timeset2[4])
	{
		AlarmPoint2_ON = Timeset2[0]*60+Timeset2[1];
		AlarmPoint2_OFF = Timeset2[2]*60+Timeset2[3];
		RelayOnOff(CurrentTimePoint, AlarmPoint2_ON, AlarmPoint2_OFF, 1);
	}
}

void RestoreTimeSet(void)
{
	uint8_t addr1 = 0x10;
	uint8_t addr2 = 0x20;
	HAL_I2C_Master_Transmit(&hi2c1, 0xD0, &addr1, 1, 0xFF);
	HAL_I2C_Master_Receive(&hi2c1, 0xD0, Timeset1, 5, 0xFF);
	HAL_I2C_Master_Transmit(&hi2c1, 0xD0, &addr2, 1, 0xFF);
	HAL_I2C_Master_Receive(&hi2c1, 0xD0, Timeset2, 5, 0xFF);
}

void EditTime (void)
{
	switch (EditModeCursor)
  {
  	case 0:
			SetValue(&DisplayTimeData[EditModeCursor], 0, 23);
	 		break;
  	case 1:
			SetValue(&DisplayTimeData[EditModeCursor], 0, 59);
  		break;
		case 2:
			SetValue(&DisplayTimeData[EditModeCursor], 0, 59);
  		break;
		case 3:
			SetValue(&DisplayTimeData[EditModeCursor], 1, 7);
  		break;
		case 4:
			SetValue(&DisplayTimeData[EditModeCursor], 1, 31);
  		break;
		case 5:
			SetValue(&DisplayTimeData[EditModeCursor], 1, 12);
  		break;
		case 6:
			SetValue(&DisplayTimeData[EditModeCursor], 17, 100);
  		break;
  	default:
			KeySelectEvent = 0;
			EditModeCursor = 0;
			EditModeItem = 0;
			// saving new time
			TimerWrite(0x02, ((DisplayTimeData[0]/10)<<4)+DisplayTimeData[0]%10); //hours
			TimerWrite(0x01, ((DisplayTimeData[1]/10)<<4)+DisplayTimeData[1]%10); //minutes
			TimerWrite(0x03, DisplayTimeData[3]); //day
			TimerWrite(0x04, ((DisplayTimeData[4]/10)<<4)+DisplayTimeData[4]%10); //date
			TimerWrite(0x05, ((DisplayTimeData[5]/10)<<4)+DisplayTimeData[5]%10); //month
			TimerWrite(0x06, ((DisplayTimeData[6]/10)<<4)+DisplayTimeData[6]%10); //year
			TimerWrite(0x00, ((DisplayTimeData[2]/10)<<4)+DisplayTimeData[2]%10);
  		break;
  }
}

void EditTimeset(uint8_t *Timeset)
{
	static uint8_t add_shift;
		switch (EditModeCursor)
  {
  	case 0:
			SetValue(&Timeset[EditModeCursor], 0, 23);
	 		break;
  	case 1:
			SetValue(&Timeset[EditModeCursor], 0, 59);
  		break;
		case 2:
			SetValue(&Timeset[EditModeCursor], 0, 23);
  		break;
		case 3:
			SetValue(&Timeset[EditModeCursor], 0, 59);
  		break;
		case 4:
			SetValue(&Timeset[EditModeCursor], 0, 1);
  		break;
  	default:
			KeySelectEvent = 0;
			EditModeCursor = 0;
			if (EditModeItem == 3) add_shift = 0x10;
			else add_shift = 0x00;
			EditModeItem = 0;
			Blink = 0;
			ForceRedraw = 1;
			DisplayTimeset();
			// saving new timeset
			TimerWrite(0x10+add_shift, Timeset[0]);
			TimerWrite(0x11+add_shift, Timeset[1]);
			TimerWrite(0x12+add_shift, Timeset[2]);
			TimerWrite(0x13+add_shift, Timeset[3]);
			TimerWrite(0x14+add_shift, Timeset[4]);
  		break;
  }
}

void SetValue(uint8_t* val, uint8_t lower, uint8_t upper)
{
	if (KeyUpEvent)
	{
		if ((*val) < upper) (*val)++;
		else (*val) = lower;
		KeyUpEvent = 0;
	}
	if (KeyDownEvent)
	{
		if ((*val) > lower) (*val)--;
		else (*val) = upper;
		KeyDownEvent = 0;
	}
}

void EventExec(void)
{
	if (KeyUpEvent) 
	{
		if (!EditModeItem) 
		{
			CursorPosition--;
			KeyUpEvent = 0;
		}
		ForceRedraw = 1;
	}
	if (KeyDownEvent) 
	{
		if (!EditModeItem) 
		{
			CursorPosition++;
			KeyDownEvent = 0;
		}
		ForceRedraw = 1;
		
	}
	if (EditModeItem && KeySelectEvent)
	{
		EditModeCursor++;
		KeySelectEvent = 0;
	}
	if (LongKeySelectEvent)
	{
		EditModeItem = CursorPosition +1;
		LongKeySelectEvent = 0;
		KeySelectEvent = 0;
	}
	
	if (EditModeItem == 1) 
	{
		TimerWrite(0x00, TimerData[0]|0x80);
		EditTime();
	}
	if (EditModeItem == 2) EditTimeset(Timeset1);
	if (EditModeItem == 3) EditTimeset(Timeset2);
}

void MenuExec(void)
{
	switch (CursorPosition)
  {
  	case 0:
			MT_WH1602_WriteData(CLEAR_DISPLAY, MT_WH1602_COMMAND);
			HAL_Delay(5);
			DisplayTime();
			break;
  	case 1:
			MT_WH1602_WriteData(CLEAR_DISPLAY, MT_WH1602_COMMAND);
			HAL_Delay(5);
			DisplayTimeset();
			MT_WH1602_WriteData(SET_DDRAM_ADDR | 0x00, MT_WH1602_COMMAND);                     
			HAL_Delay(1);
			lcd_write_str(">");
  		break;
		case 2:
			MT_WH1602_WriteData(CLEAR_DISPLAY, MT_WH1602_COMMAND);
			HAL_Delay(5);
			DisplayTimeset();
			MT_WH1602_WriteData(SET_DDRAM_ADDR | 0x40, MT_WH1602_COMMAND);                     
			HAL_Delay(1);
			lcd_write_str(">");
  		break;
  	default:
			CursorPosition = OldCursorPosition;
  		break;
  }
	OldCursorPosition = CursorPosition;
}

void Timer_read(void)
{
	static uint8_t addr = 0;
	HAL_I2C_Master_Transmit(&hi2c1, 0xD0, &addr, 1, 0xFF);
	HAL_I2C_Master_Receive(&hi2c1, 0xD0, TimerData, 8, 0xFF);
}

void TimerWrite(uint8_t addr, uint8_t data)
{
	uint8_t tbuf[2] = {addr, data};
	HAL_I2C_Master_Transmit(&hi2c1, 0xD0, tbuf, 2, 0xFF);
}

void DisplayTime(void)
{
	static uint8_t OldTimerData;
	char str_buf[17];

	if (TimerData[0]!=OldTimerData || ForceRedraw)
	{
		if (EditModeItem != 1)
		{
			OldTimerData = TimerData[0];
			DisplayTimeData[0] = ((TimerData[2]&0x30)>>4)*10 + (TimerData[2]&0x0F); //hours
			DisplayTimeData[1] = ((TimerData[1]&0x70)>>4)*10 + (TimerData[1]&0x0F);	//minutes
			DisplayTimeData[2] = ((TimerData[0]&0x70)>>4)*10 + (TimerData[0]&0x0F);	//seconds
			DisplayTimeData[3] = TimerData[3]&0x07;																	//day of week
			DisplayTimeData[4] = ((TimerData[4]&0x30)>>4)*10 + (TimerData[4]&0x0F);	//date
			DisplayTimeData[5] = ((TimerData[5]&0x10)>>4)*10 + (TimerData[5]&0x0F);	//month
			DisplayTimeData[6] = ((TimerData[6]&0xF0)>>4)*10 + (TimerData[6]&0x0F);	//year
		}
		
		sprintf(str_buf, "  %02u:%02u:%02u ", DisplayTimeData[0], DisplayTimeData[1], DisplayTimeData[2]);
		MT_WH1602_WriteData(SET_DDRAM_ADDR | 0x00, MT_WH1602_COMMAND);
		HAL_Delay(1);
		lcd_write_str(str_buf);
		
		
		
		switch (DisplayTimeData[3])
    {
    	case 1:
				lcd_write_str("MON");
    		break;
    	case 2:
				lcd_write_str("TUE");
    		break;
			case 3:
				lcd_write_str("WED");
    		break;
			case 4:
				lcd_write_str("THU");
    		break;
			case 5:
				lcd_write_str("FRI");
    		break;
			case 6:
				lcd_write_str("SAT");
    		break;
			case 7:
				lcd_write_str("SUN");
    		break;
    	default:
    		break;
    }
		
		sprintf(str_buf, "   %02u.%02u.%04u", DisplayTimeData[4], DisplayTimeData[5], DisplayTimeData[6]+2000);
		MT_WH1602_WriteData(SET_DDRAM_ADDR | 0x40, MT_WH1602_COMMAND);
		HAL_Delay(1);
		lcd_write_str(str_buf);		
	}
	
	if (Blink) 
	{
		uint8_t local_addr;
		switch (EditModeCursor)
		{
			case 0: local_addr = 0x02;
				break;
			case 1: local_addr = 0x05;
				break;
			case 2: local_addr = 0x08;
				break;
			case 3: local_addr = 0x0B;
				break;
			case 4: local_addr = 0x43;
				break;
			case 5: local_addr = 0x46;
				break;
			case 6: local_addr = 0x49;
				break;
			default:
				break;
		}
		MT_WH1602_WriteData(SET_DDRAM_ADDR | local_addr, MT_WH1602_COMMAND);
		HAL_Delay(1);
		switch (EditModeCursor)
		{
			case 0: lcd_write_str("  ");
				break;
			case 1: lcd_write_str("  ");
				break;
			case 2: lcd_write_str("  ");
				break;
			case 3: lcd_write_str("   ");
				break;
			case 4: lcd_write_str("  ");
				break;
			case 5: lcd_write_str("  ");
				break;
			case 6: lcd_write_str("    ");
				break;
			default:
				break;
		}
	}
	ForceRedraw = 0;
	TimeBlink();
}

void TimeBlink(void)
{
	static uint32_t blink_timeout;
	if (EditModeItem)
	{
		if (HAL_GetTick() > blink_timeout)
		{
			blink_timeout = HAL_GetTick() + BLINK_TIMEOUT;
			if (Blink) Blink = 0;
			else Blink = 1;
			ForceRedraw = 1;
		}	
	}
	else
	{
		blink_timeout = 0;
		Blink = 0;
		ForceRedraw = 1;
	}
}

void DisplayTimeset(void)
{
	char str_buf[13];
	static uint8_t add_shift;
	
	if (ForceRedraw)
	{
		sprintf(str_buf, "%02u:%02u-%02u:%02u ", Timeset1[0], Timeset1[1], Timeset1[2], Timeset1[3]);
		MT_WH1602_WriteData(SET_DDRAM_ADDR | 0x01, MT_WH1602_COMMAND);
		HAL_Delay(1);
		lcd_write_str(str_buf);
		if (Timeset1[4]) lcd_write_str("ON ");
		else lcd_write_str("OFF");
		
		sprintf(str_buf, "%02u:%02u-%02u:%02u ", Timeset2[0], Timeset2[1], Timeset2[2], Timeset2[3]);
		MT_WH1602_WriteData(SET_DDRAM_ADDR | 0x41, MT_WH1602_COMMAND);
		HAL_Delay(1);
		lcd_write_str(str_buf);
		if (Timeset2[4]) lcd_write_str("ON ");
		else lcd_write_str("OFF");
	}

	if (EditModeItem == 3) add_shift = 0x40;
	else add_shift = 0;
	
	if (Blink) 
	{
		uint8_t local_addr;
		switch (EditModeCursor)
		{
			case 0: local_addr = 0x01+add_shift;
				break;
			case 1: local_addr = 0x04+add_shift;
				break;
			case 2: local_addr = 0x07+add_shift;
				break;
			case 3: local_addr = 0x0A+add_shift;
				break;
			case 4: local_addr = 0x0D+add_shift;
				break;
			default:
				break;
		}
		MT_WH1602_WriteData(SET_DDRAM_ADDR | local_addr, MT_WH1602_COMMAND);
		HAL_Delay(1);
		switch (EditModeCursor)
		{
			case 0: lcd_write_str("  ");
				break;
			case 1: lcd_write_str("  ");
				break;
			case 2: lcd_write_str("  ");
				break;
			case 3: lcd_write_str("  ");
				break;
			case 4: lcd_write_str("   ");
				break;
			default:
				break;
		}
	}
	ForceRedraw = 0;
	TimeBlink();
}

void KeyboardScan(void)
{
	static uint8_t OldKeyUp = 1, OldKeyDown = 1, OldKeySelect = 1;
	static uint8_t KeyUpState = 1, KeyDownState = 1, KeySelectState = 1;
	static uint32_t timeout;
	static uint32_t long_timeout;
	
	if (!timeout)
	{
		KeyUpState			= HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_1);
		KeyDownState		= HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_2);
		KeySelectState	= HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_0);
	}
	
	if (KeyUpState != OldKeyUp && !timeout)
	{
		OldKeyUp = KeyUpState;
		if (!KeyUpState) KeyUpEvent = 1;
		timeout = HAL_GetTick() + BUTTON_TIMEOUT;
	}
	
	if (KeyDownState != OldKeyDown && !timeout)
	{
		OldKeyDown = KeyDownState;
		if (!KeyDownState) KeyDownEvent = 1;
		timeout = HAL_GetTick() + BUTTON_TIMEOUT;
	}
	
	if (KeySelectState != OldKeySelect && !timeout)
	{
		OldKeySelect = KeySelectState;
		if (!KeySelectState) KeySelectEvent = 1;
		if (KeySelectState) long_timeout = 0;
		timeout = HAL_GetTick() + BUTTON_TIMEOUT;
	}
	
	if (!KeySelectState && !timeout && !long_timeout)
	{
		long_timeout = HAL_GetTick() + LONG_PRESS_TIMEOUT;
	}
	
	if (timeout && HAL_GetTick()>=timeout) timeout = 0;
	
	if (long_timeout && (HAL_GetTick()>=long_timeout))
	{
		long_timeout = 0;
		LongKeySelectEvent = 1;
	}
}
/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInit;

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;

  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0);


  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_I2C1;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_HSI;
  HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit);

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* I2C1 init function */
static void MX_I2C1_Init(void)
{

  hi2c1.Instance = I2C1;
  hi2c1.Init.Timing = 0x2000090E;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  HAL_I2C_Init(&hi2c1);

    /**Configure Analogue filter 
    */
  HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE);

}

static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOF_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOF, REL1_Pin|REL2_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_3|GPIO_PIN_4|GPIO_PIN_5|GPIO_PIN_6 
                          |GPIO_PIN_7, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_1, GPIO_PIN_RESET);

  /*Configure GPIO pins : REL1_Pin REL2_Pin */
  GPIO_InitStruct.Pin = REL1_Pin|REL2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOF, &GPIO_InitStruct);

  /*Configure GPIO pins : PA0 PA1 PA2 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_2;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PA3 PA4 PA5 PA6 PA7 */
  GPIO_InitStruct.Pin = GPIO_PIN_3|GPIO_PIN_4|GPIO_PIN_5|GPIO_PIN_6 
                          |GPIO_PIN_7;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : PB1 */
  GPIO_InitStruct.Pin = GPIO_PIN_1;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

}

void Error_Handler(void){while(1){}}

